/*  XMMS - Cross-platform multimedia player
 *  Copyright (C) 1998-1999  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
 *
 * Fishmatic Plugin
 * Copyright (C) 1999 Rick Haines
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <gtk/gtk.h>
#include <stdio.h>
#include <stdlib.h>

#include "xmms/configfile.h"
#include "fishmatic_plugin.h"

static GtkWidget *configure_win = NULL;
static GtkWidget *vbox, *options_frame, *options_vbox;
static GtkWidget *iyff_label;
static GtkWidget *stereo_label, *stereo_slider;
static GtkObject *stereo_slider_adj;
static GtkWidget *speed_label, *speed_slider;
static GtkObject *speed_slider_adj;
static GtkWidget *count_label, *count_slider;
static GtkObject *count_slider_adj;

static GtkWidget *bbox, *ok;


static void configure_ok(GtkWidget *widget, gpointer data)
{
  fishmatic_write_config();
  gtk_widget_destroy(configure_win);
}

static void speed_slider_changed(GtkWidget *widget, gpointer data)
{
    // fishmatic_cfg.tessellation_level = (int) GTK_ADJUSTMENT(tess_slider_adj)->value;
    fishmatic_cfg.fish_speed = GTK_ADJUSTMENT(speed_slider_adj)->value;
}

static void stereo_slider_changed(GtkWidget *widget, gpointer data)
{
    // fishmatic_cfg.tessellation_level = (int) GTK_ADJUSTMENT(tess_slider_adj)->value;
    fishmatic_cfg.stereo_repeat = (int)GTK_ADJUSTMENT(stereo_slider_adj)->value;
}

static void count_slider_changed(GtkWidget *widget, gpointer data)
{
    // fishmatic_cfg.tessellation_level = (int) GTK_ADJUSTMENT(tess_slider_adj)->value;
    fishmatic_cfg.fish_count = (int)GTK_ADJUSTMENT(count_slider_adj)->value;
}

void fishmatic_configure(void)
{
  if (configure_win) return;

  fishmatic_read_config();

  configure_win = gtk_window_new(GTK_WINDOW_DIALOG);
  gtk_container_set_border_width(GTK_CONTAINER(configure_win), 10);
  gtk_window_set_title(GTK_WINDOW(configure_win), 
		       "Fishmatic Config");
  gtk_window_set_policy(GTK_WINDOW(configure_win), FALSE, FALSE, FALSE);
  gtk_window_set_position(GTK_WINDOW(configure_win), GTK_WIN_POS_MOUSE);
  gtk_signal_connect(GTK_OBJECT(configure_win), "destroy",
		     GTK_SIGNAL_FUNC(gtk_widget_destroyed), &configure_win);

  vbox = gtk_vbox_new(FALSE, 5);

  iyff_label = gtk_label_new("In Your Face Fish!");
  gtk_widget_show(iyff_label);
  gtk_box_pack_start(GTK_BOX(vbox), iyff_label, FALSE, FALSE, 0);

  options_frame = gtk_frame_new("Options:");
  gtk_container_set_border_width(GTK_CONTAINER(options_frame), 5);

  options_vbox = gtk_vbox_new(FALSE, 5);
  gtk_container_set_border_width(GTK_CONTAINER(options_vbox), 5);





  stereo_label = gtk_label_new("Stereo Repeats:");
  gtk_box_pack_start(GTK_BOX(options_vbox), stereo_label, FALSE, FALSE, 0);
  gtk_widget_show(stereo_label);

  stereo_slider_adj = gtk_adjustment_new(fishmatic_cfg.stereo_repeat,
				       4, 8, 1, 1, 0);
  gtk_signal_connect(GTK_OBJECT(stereo_slider_adj), "value_changed", 
		     GTK_SIGNAL_FUNC(stereo_slider_changed), NULL);
  stereo_slider = gtk_hscale_new(GTK_ADJUSTMENT(stereo_slider_adj));
  gtk_scale_set_digits(GTK_SCALE(stereo_slider), 0);
  gtk_box_pack_start(GTK_BOX(options_vbox), stereo_slider, FALSE, FALSE, 0);
  gtk_widget_show(stereo_slider);




  speed_label = gtk_label_new("Fishy Speed:");
  gtk_box_pack_start(GTK_BOX(options_vbox), speed_label, FALSE, FALSE, 0);
  gtk_widget_show(speed_label);

  speed_slider_adj = gtk_adjustment_new(fishmatic_cfg.fish_speed,
				       0.2, 4.0, 1, 1, 0);
  gtk_signal_connect(GTK_OBJECT(speed_slider_adj), "value_changed", 
		     GTK_SIGNAL_FUNC(speed_slider_changed), NULL);
  speed_slider = gtk_hscale_new(GTK_ADJUSTMENT(speed_slider_adj));
  gtk_box_pack_start(GTK_BOX(options_vbox), speed_slider, FALSE, FALSE, 0);
  gtk_widget_show(speed_slider);



  count_label = gtk_label_new("Number of fish:");
  gtk_box_pack_start(GTK_BOX(options_vbox), count_label, FALSE, FALSE, 0);
  gtk_widget_show(count_label);

  count_slider_adj = gtk_adjustment_new(fishmatic_cfg.fish_count,
				       1, 8, 1, 1, 0);
  gtk_signal_connect(GTK_OBJECT(count_slider_adj), "value_changed", 
		     GTK_SIGNAL_FUNC(count_slider_changed), NULL);
  count_slider = gtk_hscale_new(GTK_ADJUSTMENT(count_slider_adj));
  gtk_scale_set_digits(GTK_SCALE(count_slider), 0);
  gtk_box_pack_start(GTK_BOX(options_vbox), count_slider, FALSE, FALSE, 0);
  gtk_widget_show(count_slider);



#if 0


  fullscreen_tog = gtk_check_button_new_with_label("Fullscreen");
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fullscreen_tog),
			       fishmatic_cfg.fullscreen);
  gtk_signal_connect(GTK_OBJECT(fullscreen_tog), "toggled",
		     GTK_SIGNAL_FUNC(fullscreen_toggled), NULL);
  gtk_box_pack_start(GTK_BOX(options_vbox), fullscreen_tog, FALSE, FALSE, 0);
  gtk_widget_show(fullscreen_tog);

  flat_shad_tog = gtk_check_button_new_with_label("Flat Shading");
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(flat_shad_tog),
			       fishmatic_cfg.flat_shading);
  gtk_signal_connect(GTK_OBJECT(flat_shad_tog), "toggled",
		     GTK_SIGNAL_FUNC(flat_shad_toggled), NULL);
  gtk_box_pack_start(GTK_BOX(options_vbox), flat_shad_tog, FALSE, FALSE, 0);
  gtk_widget_show(flat_shad_tog);

  draw_grnd_tog = gtk_check_button_new_with_label("Draw Ground");
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(draw_grnd_tog),
			       fishmatic_cfg.draw_ground);
  gtk_signal_connect(GTK_OBJECT(draw_grnd_tog), "toggled",
		     GTK_SIGNAL_FUNC(draw_grnd_toggled), NULL);
  gtk_box_pack_start(GTK_BOX(options_vbox), draw_grnd_tog, FALSE, FALSE, 0);
  gtk_widget_show(draw_grnd_tog);

  tess_label = gtk_label_new("Tessellation Level:");
  gtk_box_pack_start(GTK_BOX(options_vbox), tess_label, FALSE, FALSE, 0);
  gtk_widget_show(tess_label);

  tess_slider_adj = gtk_adjustment_new(fishmatic_cfg.tessellation_level,
				       1.0, 256.0, 1.0, 8.0, 8.0);
  gtk_signal_connect(GTK_OBJECT(tess_slider_adj), "value_changed", 
		     GTK_SIGNAL_FUNC(tess_slider_changed), NULL);
  tess_slider = gtk_hscale_new(GTK_ADJUSTMENT(tess_slider_adj));
  gtk_box_pack_start(GTK_BOX(options_vbox), tess_slider, FALSE, FALSE, 0);
  gtk_widget_show(tess_slider);

  cam_vel_label = gtk_label_new("Camera Velocity:");
  gtk_box_pack_start(GTK_BOX(options_vbox), cam_vel_label, FALSE, FALSE, 0);
  gtk_widget_show(cam_vel_label);

  cam_vel_slider_adj = gtk_adjustment_new(fishmatic_cfg.camera_velocity,
				       0.0, 40.0, 1.0, 8.0, 8.0);
  gtk_signal_connect(GTK_OBJECT(cam_vel_slider_adj), "value_changed", 
		     GTK_SIGNAL_FUNC(cam_vel_slider_changed), NULL);
  cam_vel_slider = gtk_hscale_new(GTK_ADJUSTMENT(cam_vel_slider_adj));
  gtk_box_pack_start(GTK_BOX(options_vbox), cam_vel_slider, FALSE, FALSE, 0);
  gtk_widget_show(cam_vel_slider);

  cam_height_label = gtk_label_new("Camera Height:");
  gtk_box_pack_start(GTK_BOX(options_vbox), cam_height_label, FALSE, FALSE, 0);
  gtk_widget_show(cam_height_label);

  cam_height_slider_adj = gtk_adjustment_new(fishmatic_cfg.camera_height,
					     -22.0, 22.0, 0.25, 2.0, 2.0);
  gtk_signal_connect(GTK_OBJECT(cam_height_slider_adj), "value_changed", 
		     GTK_SIGNAL_FUNC(cam_height_slider_changed), NULL);
  cam_height_slider = gtk_hscale_new(GTK_ADJUSTMENT(cam_height_slider_adj));
  gtk_box_pack_start(GTK_BOX(options_vbox), cam_height_slider, FALSE, FALSE, 0);
  gtk_widget_show(cam_height_slider);

#endif
  gtk_container_add(GTK_CONTAINER(options_frame), options_vbox);
  gtk_widget_show(options_vbox);

  gtk_box_pack_start(GTK_BOX(vbox), options_frame, TRUE, TRUE, 0);
  gtk_widget_show(options_frame);


  bbox = gtk_hbutton_box_new();
  gtk_button_box_set_layout(GTK_BUTTON_BOX(bbox), GTK_BUTTONBOX_END);
  gtk_button_box_set_spacing(GTK_BUTTON_BOX(bbox), 5);
  gtk_box_pack_start(GTK_BOX(vbox), bbox, FALSE, FALSE, 0);


  ok = gtk_button_new_with_label("Ok");
  gtk_signal_connect(GTK_OBJECT(ok), "clicked", GTK_SIGNAL_FUNC(configure_ok), NULL);

  GTK_WIDGET_SET_FLAGS(ok, GTK_CAN_DEFAULT);
  gtk_box_pack_start(GTK_BOX(bbox), ok, TRUE, TRUE, 0);
  gtk_widget_show(ok);

#if 0
  cancel = gtk_button_new_with_label("Cancel");
  gtk_signal_connect_object(GTK_OBJECT(cancel), "clicked",
			    GTK_SIGNAL_FUNC(gtk_widget_destroy),
			    GTK_OBJECT(configure_win));
  GTK_WIDGET_SET_FLAGS(cancel, GTK_CAN_DEFAULT);
  gtk_box_pack_start(GTK_BOX(bbox), cancel, TRUE, TRUE, 0);
  gtk_widget_show(cancel);
#endif
  gtk_widget_show(bbox);

  gtk_container_add(GTK_CONTAINER(configure_win), vbox);

  gtk_widget_show(vbox);
  gtk_widget_show(configure_win);
  gtk_widget_grab_default(ok);
}

